@@ -1,5 +1,5 @@
 
-/* Copyright (c) 1999-2016 by The D Language Foundation
+/* Copyright (C) 1999-2018 by The D Language Foundation, All Rights Reserved
  * All Rights Reserved, written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -13,13 +13,7 @@
 #include "longdouble.h"
 
 // Type used by the front-end for compile-time reals
-#if IN_LLVM && _MSC_VER
-// Make sure LDC built with MSVC uses double-precision compile-time reals,
-// independent of whether it was built with DMD (80-bit reals) or LDC.
-typedef double real_t;
-#else
 typedef longdouble real_t;
-#endif
 
 #if IN_LLVM
 namespace llvm { class APFloat; }
@@ -41,20 +35,27 @@ struct CTFloat
     static real_t fabs(real_t x);
     static real_t ldexp(real_t n, int exp);
 
-#if IN_LLVM
-    static real_t log(real_t x);
-    static real_t log2(real_t x);
-    static real_t log10(real_t x);
-    static real_t fmin(real_t l, real_t r);
-    static real_t fmax(real_t l, real_t r);
+    static real_t round(real_t x);
     static real_t floor(real_t x);
     static real_t ceil(real_t x);
     static real_t trunc(real_t x);
+    static real_t log(real_t x);
+    static real_t log2(real_t x);
+    static real_t log10(real_t x);
+    static real_t pow(real_t x, real_t y);
+    static real_t expm1(real_t x);
+    static real_t exp2(real_t x);
+
+    static real_t fmin(real_t x, real_t y);
+    static real_t fmax(real_t x, real_t y);
+    static real_t copysign(real_t x, real_t s);
+
+    static real_t fma(real_t x, real_t y, real_t z);
+
+#if IN_LLVM
+    static real_t exp(real_t x);
     static real_t rint(real_t x);
     static real_t nearbyint(real_t x);
-    static real_t round(real_t x);
-    static real_t fma(real_t x, real_t y, real_t z);
-    static real_t copysign(real_t to, real_t from);
 
     // implemented in gen/ctfloat.cpp
     static void _init();
