@@ -1,5 +1,5 @@
 
-/* Copyright (c) 1999-2016 by The D Language Foundation
+/* Copyright (C) 1999-2018 by The D Language Foundation, All Rights Reserved
  * All Rights Reserved, written by Walter Bright
  * http://www.digitalmars.com
  * Distributed under the Boost Software License, Version 1.0.
@@ -11,7 +11,7 @@
 #define OBJECT_H
 
 #if !IN_LLVM
-#define POSIX (__linux__ || __APPLE__ || __FreeBSD__ || __OpenBSD__ || __sun)
+#define POSIX (__linux__ || __APPLE__ || __FreeBSD__ || __OpenBSD__ || __DragonFly__ || __sun)
 #endif
 
 #if __DMC__
