@@ -65,11 +65,6 @@ void DtoResolveClass(ClassDeclaration *cd) {
     getIrField(vd, true);
   }
 
-  // emit the interfaceInfosZ symbol if necessary
-  if (cd->vtblInterfaces && cd->vtblInterfaces->dim > 0) {
-    irAggr->getInterfaceArraySymbol(); // initializer is applied when it's built
-  }
-
   // interface only emit typeinfo and classinfo
   if (cd->isInterfaceDeclaration()) {
     irAggr->initializeInterface();
@@ -100,7 +95,7 @@ DValue *DtoNewClass(Loc &loc, TypeClass *tc, NewExp *newexp) {
     llvm::Function *fn =
         getRuntimeFunction(loc, gIR->module, "_d_allocclass");
     LLConstant *ci = DtoBitCast(getIrAggr(tc->sym)->getClassInfoSymbol(),
-                                DtoType(Type::typeinfoclass->type));
+                                DtoType(getClassInfoType()));
     mem =
         gIR->CreateCallOrInvoke(fn, ci, ".newclass_gc_alloc").getInstruction();
     mem = DtoBitCast(mem, DtoType(tc), ".newclass_gc");
@@ -347,17 +342,25 @@ DValue *DtoCastClass(Loc &loc, DValue *val, Type *_to) {
 
 ////////////////////////////////////////////////////////////////////////////////
 
+static void resolveObjectAndClassInfoClasses() {
+  // check declarations in object.d
+  getObjectType();
+  getClassInfoType();
+
+  DtoResolveClass(ClassDeclaration::object);
+  DtoResolveClass(Type::typeinfoclass);
+}
+
 DValue *DtoDynamicCastObject(Loc &loc, DValue *val, Type *_to) {
   // call:
   // Object _d_dynamic_cast(Object o, ClassInfo c)
 
-  DtoResolveClass(ClassDeclaration::object);
-  DtoResolveClass(Type::typeinfoclass);
-
   llvm::Function *func =
       getRuntimeFunction(loc, gIR->module, "_d_dynamic_cast");
   LLFunctionType *funcTy = func->getFunctionType();
 
+  resolveObjectAndClassInfoClasses();
+
   // Object o
   LLValue *obj = DtoRVal(val);
   obj = DtoBitCast(obj, funcTy->getParamType(0));
@@ -389,13 +392,12 @@ DValue *DtoDynamicCastInterface(Loc &loc, DValue *val, Type *_to) {
   // call:
   // Object _d_interface_cast(void* p, ClassInfo c)
 
-  DtoResolveClass(ClassDeclaration::object);
-  DtoResolveClass(Type::typeinfoclass);
-
   llvm::Function *func =
       getRuntimeFunction(loc, gIR->module, "_d_interface_cast");
   LLFunctionType *funcTy = func->getFunctionType();
 
+  resolveObjectAndClassInfoClasses();
+
   // void* p
   LLValue *ptr = DtoRVal(val);
   ptr = DtoBitCast(ptr, funcTy->getParamType(0));
@@ -522,7 +524,7 @@ static LLConstant *build_offti_array(ClassDeclaration *cd, LLType *arrayT) {
 #endif // GENERATE_OFFTI
 
 static LLConstant *build_class_dtor(ClassDeclaration *cd) {
-  FuncDeclaration *dtor = cd->dtor;
+  FuncDeclaration *dtor = cd->tidtor;
 
   // if no destructor emit a null
   if (!dtor) {
@@ -606,7 +608,8 @@ LLConstant *DtoDefineClassInfo(ClassDeclaration *cd) {
   assert(cd->type->ty == Tclass);
 
   IrAggr *ir = getIrAggr(cd);
-  ClassDeclaration *cinfo = Type::typeinfoclass;
+  Type *const cinfoType = getClassInfoType(); // check declaration in object.d
+  ClassDeclaration *const cinfo = Type::typeinfoclass;
 
   if (cinfo->fields.dim != 12) {
     error(Loc(), "Unexpected number of fields in `object.ClassInfo`; "
@@ -615,7 +618,7 @@ LLConstant *DtoDefineClassInfo(ClassDeclaration *cd) {
   }
 
   // use the rtti builder
-  RTTIBuilder b(cinfo);
+  RTTIBuilder b(cinfoType);
 
   LLConstant *c;
 
@@ -656,7 +659,7 @@ LLConstant *DtoDefineClassInfo(ClassDeclaration *cd) {
   if (cd->baseClass && !cd->isInterfaceDeclaration()) {
     b.push_classinfo(cd->baseClass);
   } else {
-    b.push_null(cinfo->type);
+    b.push_null(cinfoType);
   }
 
   // destructor
