@@ -911,17 +911,12 @@ void DtoResolveVariable(VarDeclaration *vd) {
     // with a different type later, swap it out and replace any existing
     // uses with bitcasts to the previous type.
 
-    // We always start out with external linkage; any other type is set
-    // when actually defining it in VarDeclaration::codegen.
-    llvm::GlobalValue::LinkageTypes linkage =
-        llvm::GlobalValue::ExternalLinkage;
-    if (vd->llvmInternal == LLVMextern_weak) {
-      linkage = llvm::GlobalValue::ExternalWeakLinkage;
-    }
-
     llvm::GlobalVariable *gvar =
-        getOrCreateGlobal(vd->loc, gIR->module, DtoMemType(vd->type), isLLConst,
-                          linkage, nullptr, irMangle, vd->isThreadlocal());
+        declareGlobal(vd->loc, gIR->module, DtoMemType(vd->type), irMangle,
+                      isLLConst, vd->isThreadlocal());
+    if (vd->llvmInternal == LLVMextern_weak)
+      gvar->setLinkage(llvm::GlobalValue::ExternalWeakLinkage);
+
     auto varIr = getIrGlobal(vd);
     varIr->value = gvar;
 
@@ -971,9 +966,10 @@ void DtoVarDeclaration(VarDeclaration *vd) {
 
   if (isIrLocalCreated(vd)) {
     // Nothing to do if it has already been allocated.
-  } else if (gIR->func()->sretArg && ((gIR->func()->decl->nrvo_can &&
-                                       gIR->func()->decl->nrvo_var == vd) ||
-                                      vd->isResult())) {
+  } else if (gIR->func()->sretArg &&
+             ((gIR->func()->decl->nrvo_can &&
+               gIR->func()->decl->nrvo_var == vd) ||
+              (vd->isResult() && !isSpecialRefVar(vd)))) {
     // Named Return Value Optimization (NRVO):
     // T f() {
     //   T ret;        // &ret == hidden pointer
@@ -982,6 +978,7 @@ void DtoVarDeclaration(VarDeclaration *vd) {
     // }
     assert(!isSpecialRefVar(vd) && "Can this happen?");
     getIrLocal(vd, true)->value = gIR->func()->sretArg;
+    gIR->DBuilder.EmitLocalVariable(gIR->func()->sretArg, vd);
   } else {
     // normal stack variable, allocate storage on the stack if it has not
     // already been done
@@ -1278,14 +1275,15 @@ LLConstant *DtoTypeInfoOf(Type *type, bool base) {
                          type->toChars(), base);
   LOG_SCOPE
 
-  TypeInfoDeclaration *tidecl = getOrCreateTypeInfoDeclaration(type, nullptr);
+  TypeInfoDeclaration *tidecl =
+      getOrCreateTypeInfoDeclaration(Loc(), type, nullptr);
   assert(tidecl);
   Declaration_codegen(tidecl);
   assert(getIrGlobal(tidecl)->value != NULL);
   LLConstant *c = isaConstant(getIrGlobal(tidecl)->value);
   assert(c != NULL);
   if (base) {
-    return llvm::ConstantExpr::getBitCast(c, DtoType(Type::dtypeinfo->type));
+    return llvm::ConstantExpr::getBitCast(c, DtoType(getTypeInfoType()));
   }
   return c;
 }
@@ -1474,33 +1472,17 @@ void tokToICmpPred(TOK op, bool isUnsigned, llvm::ICmpInst::Predicate *outPred,
                    llvm::Value **outConst) {
   switch (op) {
   case TOKlt:
-  case TOKul:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_ULT : llvm::ICmpInst::ICMP_SLT;
     break;
   case TOKle:
-  case TOKule:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_ULE : llvm::ICmpInst::ICMP_SLE;
     break;
   case TOKgt:
-  case TOKug:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_UGT : llvm::ICmpInst::ICMP_SGT;
     break;
   case TOKge:
-  case TOKuge:
     *outPred = isUnsigned ? llvm::ICmpInst::ICMP_UGE : llvm::ICmpInst::ICMP_SGE;
     break;
-  case TOKue:
-    *outPred = llvm::ICmpInst::ICMP_EQ;
-    break;
-  case TOKlg:
-    *outPred = llvm::ICmpInst::ICMP_NE;
-    break;
-  case TOKleg:
-    *outConst = LLConstantInt::getTrue(gIR->context());
-    break;
-  case TOKunord:
-    *outConst = LLConstantInt::getFalse(gIR->context());
-    break;
   default:
     llvm_unreachable("Invalid comparison operation");
   }
@@ -1593,6 +1575,14 @@ DValue *DtoSymbolAddress(Loc &loc, Type *type, Declaration *decl) {
       }
       return new DImValue(type, m);
     }
+    // special vtbl symbol, used by LDC as alias to the actual vtbl (with
+    // different type and mangled name)
+    if (vd->isClassMember() && vd == vd->isClassMember()->vtblsym) {
+      Logger::println("vtbl symbol");
+      auto cd = vd->isClassMember();
+      return new DLValue(
+          type, DtoBitCast(getIrAggr(cd)->getVtblSymbol(), DtoPtrToType(type)));
+    }
     // nested variable
     if (vd->nestedrefs.dim) {
       Logger::println("nested variable");
@@ -1750,19 +1740,32 @@ llvm::Constant *buildStringLiteralConstant(StringExp *se, bool zeroTerm) {
   return LLConstantArray::get(at, vals);
 }
 
-llvm::GlobalVariable *getOrCreateGlobal(const Loc &loc, llvm::Module &module,
-                                        llvm::Type *type, bool isConstant,
-                                        llvm::GlobalValue::LinkageTypes linkage,
-                                        llvm::Constant *init,
-                                        llvm::StringRef name,
-                                        bool isThreadLocal) {
-  llvm::GlobalVariable *existing = module.getGlobalVariable(name, true);
+static std::string llvmTypeToString(llvm::Type *type) {
+  std::string result;
+  llvm::raw_string_ostream stream(result);
+  stream << *type;
+  stream.flush();
+  return result;
+}
+
+llvm::GlobalVariable *declareGlobal(const Loc &loc, llvm::Module &module,
+                                    llvm::Type *type,
+                                    llvm::StringRef mangledName,
+                                    bool isConstant, bool isThreadLocal) {
+  llvm::GlobalVariable *existing =
+      module.getGlobalVariable(mangledName, /*AllowInternal=*/true);
   if (existing) {
-    if (existing->getType()->getElementType() != type) {
+    const auto existingType = existing->getType()->getElementType();
+    if (existingType != type || existing->isConstant() != isConstant ||
+        existing->isThreadLocal() != isThreadLocal) {
+      const auto existingTypeName = llvmTypeToString(existingType);
+      const auto newTypeName = llvmTypeToString(type);
       error(loc,
             "Global variable type does not match previous declaration with "
             "same mangled name: `%s`",
-            name.str().c_str());
+            mangledName.str().c_str());
+      errorSupplemental(loc, "Previous IR type: %s", existingTypeName.c_str());
+      errorSupplemental(loc, "New IR type:      %s", newTypeName.c_str());
       fatal();
     }
     return existing;
@@ -1771,14 +1774,38 @@ llvm::GlobalVariable *getOrCreateGlobal(const Loc &loc, llvm::Module &module,
   // Use a command line option for the thread model.
   // On PPC there is only local-exec available - in this case just ignore the
   // command line.
-  const llvm::GlobalVariable::ThreadLocalMode tlsModel =
+  const auto tlsModel =
       isThreadLocal
           ? (global.params.targetTriple->getArch() == llvm::Triple::ppc
                  ? llvm::GlobalVariable::LocalExecTLSModel
                  : clThreadModel.getValue())
           : llvm::GlobalVariable::NotThreadLocal;
-  return new llvm::GlobalVariable(module, type, isConstant, linkage, init, name,
-                                  nullptr, tlsModel);
+
+  return new llvm::GlobalVariable(module, type, isConstant,
+                                  llvm::GlobalValue::ExternalLinkage, nullptr,
+                                  mangledName, nullptr, tlsModel);
+}
+
+void defineGlobal(llvm::GlobalVariable *global, llvm::Constant *init,
+                  Dsymbol *symbolForLinkage) {
+  assert(global->isDeclaration() && "Global variable already defined");
+  assert(init);
+  global->setInitializer(init);
+  if (symbolForLinkage)
+    setLinkage(symbolForLinkage, global);
+}
+
+llvm::GlobalVariable *defineGlobal(const Loc &loc, llvm::Module &module,
+                                   llvm::StringRef mangledName,
+                                   llvm::Constant *init,
+                                   llvm::GlobalValue::LinkageTypes linkage,
+                                   bool isConstant, bool isThreadLocal) {
+  assert(init);
+  auto global = declareGlobal(loc, module, init->getType(), mangledName,
+                              isConstant, isThreadLocal);
+  defineGlobal(global, init, nullptr);
+  global->setLinkage(linkage);
+  return global;
 }
 
 FuncDeclaration *getParentFunc(Dsymbol *sym) {
@@ -1786,11 +1813,11 @@ FuncDeclaration *getParentFunc(Dsymbol *sym) {
     return nullptr;
   }
 
-  // Static functions and function (not delegate) literals don't allow
-  // access to a parent context, even if they are nested.
+  // Static/non-extern(D) functions and function (not delegate) literals don't
+  // allow access to a parent context, even if they are nested.
   if (FuncDeclaration *fd = sym->isFuncDeclaration()) {
     bool certainlyNewRoot =
-        fd->isStatic() ||
+        fd->isStatic() || fd->linkage != LINKd ||
         (fd->isFuncLiteralDeclaration() &&
          static_cast<FuncLiteralDeclaration *>(fd)->tok == TOKfunction);
     if (certainlyNewRoot) {
